;
;	>>> SV2K17 Invitro-Globals by JAC! <<<
;
;	Created for Sillyventure 2017 in Gdansk.
;
;	@com.wudsn.ide.asm.hardware=ATARI2600
;	@com.wudsn.ide.asm.mainsourcefile=SV2K17.asm

	icl "VCS.asm"

	opt l+

; Zeropage variable declaration
		org $80		;Global zeropage variables

sound_zp	.ds 9		;9 bytes stable, 2 bytes volatile
stage_cnt	.ds 1		;1 byte, counting step 2, bit 7 set by subroutine to indicate end of stage
volatile_zp	.ds 14		;Volatile ZP, maximum (11+2 for animate_ptr) is used by scroll currently
animate_ptr	= volatile_zp	;2 bytes

stable_zp	.ds zp_end-*

;audv0_ram	= stable_zp
;audf0_ram	= stable_zp+2
rink_stable_zp	= stable_zp+4
scroll_zp	= stable_zp
logo_zp		= stable_zp

zp_end		= $fa

;===============================================================

	.enum mmu			;MMU bank switching locations for 16k cartridges
		bank0 = $fff6		;Any access activates bank 0
		bank1 = $fff7		;Any access activates bank 1
		bank2 = $fff8		;Any access activates bank 2
		bank3 = $fff9		;Any access activates bank 3
	.ende

	.macro m_align_page
	.if :0=0
	.error "No alignment label specified."
	.endif

	here = *
	size = [[[here+255]/256]*256]-*
	.print "Alignment for :1 added at ",here,": ", size, " bytes."
	.rept size
	.byte 0
	.endr
	.endm

	.macro m_check_ntsc		;IN: swchb, OUT: <Z>=0 for PAL, <Z>=1 for NTSC
	lda swchb			;D3 color - B/W 0 = B/W, 1 = color
	and #8
	.endm

;===============================================================

	.macro m_bank_start		;Start new memory bank
	opt h-
	opt f-			
	org $1000+(:1)*$2000
	opt f+

	.print "==============================================================="
	.print "Start of bank :1"
	.print "==============================================================="
	
	.endm

;===============================================================

	.macro m_bank_end		;End memory bank
	bank_end_offset = $ff0

bank_end
	.print "PROC bank:1: ",  bank:1, " - ", bank_end-1, " (", bank_end-bank:1, " bytes used, ", bank:1+bank_end_offset-bank_end, " bytes free)"
	.if bank_end > bank:1+bank_end_offset
		.error "Bank overflow in bank :1 with bank_end ",bank_end
	.endif


	org bank:1+bank_end_offset
start
	bit mmu.bank0			;Activate bank0
	.if :1 = 0
		jmp bank0.main		;Start main part from bank0
	.else
		jmp *			;Unrechable code, used as indicator
	.endif

	org bank:1+$ff6			;Bank switching hot spots
	.byte 0,0,0,0			;4 banks

	org bank:1+$ffc			;Cartridge vectors
	.word start			;Reset vector
	.word start			;IRQ vector, used for BRK instruction

;	.endp				;End of bank:1

	.endm
