
;	>>> SV2K17 Invitro - Logo by JAC! <<<
;
;	Created for Sillyventure 2011 in Gdansk.
;	Reworked for Sillyventure 2017 in Gdansk.
;
;	This module requires 2 volatile bytes.
;
;	@com.wudsn.ide.asm.hardware=ATARI2600
;	@com.wudsn.ide.asm.mainsourcefile=../SV2K17.asm

	.proc logo

columns		= 8		;Number of colunmns (bytes)
rows		= 7		;Number of rows (block lines), last line is unused/empty
picture_bpl	= 64		;Bytes per line in the picture
black_columns	= 11		;Number of leading and trailing black column

ysize_max	= 21		;Number of scalines per row, actual value is +1
ysize_height	= rows*(ysize_max+1)+2		
border_color	= 14		;Color for upper and lower border of logo

cnt		= logo_zp+0
flash_cnt	= logo_zp+1
mode		= logo_zp+2	;Solid, Flash, Wave and Scale
xpos		= logo_zp+3	;X position
xsoft		= logo_zp+4	;Number of cycles to delay, computed during vblank, used during playfield, 1 byte
ysize		= logo_zp+5	;Y start position, 1 byte
ypad1		= logo_zp+6	;Upper padding height, 1 byte
ypad2		= logo_zp+7	;Lower padding height, 1 byte

sm_bpc		= rows		;SM bytes per column
sm_size		= columns*rows	;SM total size
sm		= ypad2+1	;sm_size bytes
sm_end		= sm+sm_size

logo_zp_end = sm_end

	.print "ZP logo_sm:	", sm, " - ", sm_end-1, " (", sm_end-sm, " bytes)"

.if logo_zp_end > zp_end
	.error "ZP logo_zp:	", logo_zp, " - ", logo_zp_end-1, " (", logo_zp_end-logo_zp, " bytes)"
.else
	.print "ZP logo_zp:	", logo_zp, " - ", logo_zp_end-1, " (", logo_zp_end-logo_zp, " bytes)"
.endif

;===============================================================

	.proc init
phase1	ldy #0
	.byte $2c
phase2	ldy #1
	.byte $2c
phase3	ldy #2
	.byte $2c
phase4	ldy #3

	lda #0
	ldx #logo_zp_end-logo_zp
clear	sta logo_zp,x
	dex
	bpl clear

	sty mode

	lda #$d0
	sta cnt			;To make sure that bouncing starts at the correct time

	lda #$f0
	sta flash_cnt
	rts
	.endp			;End of init

;===============================================================

	.proc vblank
flash_and = volatile_zp+0
flash_idx = volatile_zp+1

	.proc animate_counters
	inc cnt
	.proc flash
	lda flash_cnt		;Decrement flash counter?
	beq no_flash
	bpl dec_flash		;If the flash value is positive, it's decremented
	cmp #$ff
	beq no_flash 
	inc flash_cnt
	.byte $2c
dec_flash
	dec flash_cnt		;If the flash value is negative, it's incremented 
no_flash
	.endp
	.endp			;End of animate_counters

	.proc animate_mode
	ldx #0
	stx flash_and
	stx flash_idx

	lda mode
	beq phase1
	cmp #1
	beq phase2
	cmp #2
	beq phase3
	ldx #3			;Once 3 is reached, stay at 3
	stx mode	
phase3
	dec flash_and

phase2	lda cnt
	lsr
	sta flash_idx
phase1
	.endp			;End of animate_mode

	.proc animate_size
	txa			;<X>==0 means mode is not 3
	beq do_max
	lda cnt
	bmi do_max
	clc
	adc #ysize_max

	and #63
	cmp #31
	bcc *+4
	eor #63

	cmp #ysize_max
	bcc no_max
do_max	lda #ysize_max
no_max	sta ysize

	lda #ysize_height
	ldx #rows
loop	clc
	sbc ysize
	dex
	bne loop
	lsr
	sta ypad1
	adc #0
	sta ypad2
	.endp			;End of animate_size

	.proc animate_logo
	ldx xsoft		;Soft scrolling and hard scrolling
	ldy xpos
	inx			;Next xsoft
	cpx #7
	bne no_xsoft
	ldx #0
	iny			;Next xpos
	cpy #picture_bpl+black_columns
	bne no_xpos
	ldy #0
	mva #$0f flash_cnt	;Start fading out

no_xpos
	lda flash_cnt		;Check fading out
	bne no_next_stage
	lda stage_cnt		;End current stage stage 
	ora #$80	
	sta stage_cnt
no_next_stage

no_xsoft
	stx xsoft
	sty xpos

	ldx #0
copy_loop
	m_check_ntsc
	beq copy_ntsc

	lda picture.pal+picture_bpl*0-black_columns,y
	jsr animate_cell
	sta sm+0,x
	lda picture.pal+picture_bpl*1-black_columns,y
	jsr animate_cell
	sta sm+1,x
	lda picture.pal+picture_bpl*2-black_columns,y
	jsr animate_cell
	sta sm+2,x
	lda picture.pal+picture_bpl*3-black_columns,y
	jsr animate_cell
	sta sm+3,x
	lda picture.pal+picture_bpl*4-black_columns,y
	jsr animate_cell
	sta sm+4,x
	lda picture.pal+picture_bpl*5-black_columns,y
	jsr animate_cell
	sta sm+5,x
	lda picture.pal+picture_bpl*6-black_columns,y
	jmp do_sm_6
copy_ntsc
	lda picture.ntsc+picture_bpl*0-black_columns,y
	jsr animate_cell
	sta sm+0,x
	lda picture.ntsc+picture_bpl*1-black_columns,y
	jsr animate_cell
	sta sm+1,x
	lda picture.ntsc+picture_bpl*2-black_columns,y
	jsr animate_cell
	sta sm+2,x
	lda picture.ntsc+picture_bpl*3-black_columns,y
	jsr animate_cell
	sta sm+3,x
	lda picture.ntsc+picture_bpl*4-black_columns,y
	jsr animate_cell
	sta sm+4,x
	lda picture.ntsc+picture_bpl*5-black_columns,y
	jsr animate_cell
	sta sm+5,x
	lda picture.ntsc+picture_bpl*6-black_columns,y
do_sm_6
	jsr animate_cell
	sta sm+6,x

	iny
	txa
	clc
	adc #rows
	tax
	cpx #rows*columns
	bne copy_loop
	rts

	.proc animate_cell
	cpy #picture_bpl+black_columns
	bcs is_black
	cpy #black_columns
	bcc is_black
	cmp #32
	bcs normal
	and flash_and
	adc flash_idx
	and #31
	cmp #16
	bcc *+4
	eor #31
normal	rts

is_black
	lda #0
	rts
	.endp			;End of animate_cell

	.endp			;End of animate_logo

	.endp			;End of vblank

;===============================================================

	.proc playfield

	sta wsync
	sta wsync

	ldx #1
	stx ctrlpf

	.proc padding_1		;Upper padding
	ldy #0
loop	sta wsync
	lda coltab.pal.upper,y
	sta colubk
	lda #$10		;7*(7*21)+3=150 pixel always there, hence 2*4=8 of 10 pixel to mask left and right
	sta pf0			;If 2 remaining pixel appear on the right and are not really visible
	iny
	dec ypad1
	bne loop
	.endp

	sta wsync
	lda flash_cnt
	and #$0f
	sta colubk
	pha

	.proc soft_sync
	sta wsync
	lda xsoft
	beq *+2		;2/3
	cmp #2
	bcc *+2
	cmp #3
	bcc *+2
	cmp #4
	bcc *+2
	cmp #5
	bcc *+2
	cmp #6
	bcc *+2
	.endp
	
	m_assert_same_page soft_sync

	jsr wait.wait20
	jsr wait.wait18

	ldy ysize
	iny
	ldx #0

	.local loop
	dey
	bpl skip1	;3+3 = 2+2+2 = 6
	inx		;2
	ldy ysize	;3
	bne skip2	;3

skip1	pha		;3
	pla		;4
skip2
	cpx #rows
	bcs exit

	lda sm+rows*0,x	;4
	sta colubk+$100	;3 = 7
	lda sm+rows*1,x
	sta colubk
	lda sm+rows*2,x
	sta colubk
	lda sm+rows*3,x
	sta colubk
	lda sm+rows*4,x
	sta colubk
	lda sm+rows*5,x
	sta colubk
	lda sm+rows*6,x
	sta colubk
	lda sm+rows*7,x
	sta colubk
	bcc loop

	.endl			;End of loop

	m_assert_same_page loop

exit	pla
	sta colubk
	sta wsync

	.proc padding_2		;Lower padding
	ldy ypad2
	bne start
loop	sta wsync
	lda coltab.pal.lower,y
	sta colubk
start	dey
	bne loop		;End of padding_2
	.endp

	lda #0
	sta wsync
	sta colubk
	sta wsync
	rts

	.endp			;End of playfield

;===============================================================

	.proc wait		;Delay a defined number of cycles
wait20	nop
wait18	nop
wait16	nop
wait14	nop
wait12	rts
	.endp			;End of wait

;===============================================================
	
; Global color table, 12 PAL colors fading from black, brightness $0 to $e and white and back, 19 bytes per color.
; Also used for NTSC.
; Must be aligned to page boundary.
	.local coltab

	.local pal
	.local upper
	.byte $00,$60,$62,$64,$66,$68,$6a,$6c,$6e,$0e,$6e,$6c,$6a,$68,$66,$64,$62,$60
	.byte $00,$40,$42,$44,$46,$48,$4a,$4c,$4e,$0e,$4e,$4c,$4a,$48,$46,$44,$42,$40
	.byte $00,$30,$32,$34,$36,$38,$3a,$3c,$3e,$0e,$3e,$3c,$3a,$38,$36,$34,$32,$30
	.byte $00,$50,$52,$54,$56,$58,$5a,$5c,$5e,$0e,$5e,$5c,$5a,$58,$56,$54,$52,$50
	.byte $00,$70,$72,$74,$76,$78,$7a,$7c,$7e,$0e,$7e,$7c,$7a,$78,$76,$74,$72,$70
	.endl

	.local lower
	.byte $00,$90,$92,$94,$96,$98,$9a,$9c,$9e,$0e,$9e,$9c,$9a,$98,$96,$94,$92,$90
	.byte $00,$b0,$b2,$b4,$b6,$b8,$ba,$bc,$be,$0e,$be,$bc,$ba,$b8,$b6,$b4,$b2,$b0
	.byte $00,$d0,$d2,$d4,$d6,$d8,$da,$dc,$de,$0e,$de,$dc,$da,$d8,$d6,$d4,$d2,$d0
	.byte $00,$c0,$c2,$c4,$c6,$c8,$ca,$cc,$ce,$0e,$ce,$cc,$ca,$c8,$c6,$c4,$c2,$c0
	.byte $00,$a0,$a2,$a4,$a6,$a8,$aa,$ac,$ae,$0e,$ae,$ac,$aa,$a8,$a6,$a4,$a2,$a0
	.endl

	.endl

	.endl

; Scroll picture, $1c0 bytes
	.local picture
	.local ntsc
	ins "SV2K17-Logo-NTSC.pic"
	.endl
	.local pal
	ins "SV2K17-Logo-PAL.pic"
	.endl
	.endl

	.endp			;End of logo

	.print "PROC logo:	", logo, " - ", logo+.len logo-1, " (", .len logo, " bytes)"

