;	>>> SV2K17 Invitro-Scroll by JAC! <<<
;
;	Created for Sillyventure 2017 in Gdansk.
;
;	16 px border | 11*12 = 132 => 128 px scroll | 16 px border
;
;	This module requires 14 volatile bytes.
;
;	@com.wudsn.ide.asm.hardware=ATARI2600
;	@com.wudsn.ide.asm.mainsourcefile=../SV2K17.asm

	.proc scroll


	.proc init
phase1
	ldy #0			;Step 2 because it's a word index
	.byte $2c
phase2
	ldy #2
	.byte $2c
phase3
	ldy #4
	.byte $2c
phase4
	ldy #6
	.byte $2c
phase5
	ldy #8

	lda #0			;Clear used ZP
	ldx #[scroll_zp_end-scroll_zp]-1
clear	sta scroll_zp,x
	dex
	bpl clear
	
	mwa #bank1.effect.scroll.text.init_phase animate_ptr
	jmp common.call_animate

	.endp			;End of init

;===============================================================

	.proc vblank

 	.proc scroll_players_left

	lda xpos_cnt		;Count xpos_cnt from 0...frame_width
	sec
	sbc #1
	scs
	adc #frame_width
	sta xpos_cnt

	ldy frame_cnt		;Count frame_cnt from 0..frame_n
	iny
	cpy #frame_n
	bne no_frame_wrap

	.proc frame_wrap	;Frames wrap around. Move xpos_cnt to right and characters to the left.

	.proc frame_wrap_xpos
	lda xpos_cnt
	sec
	sbc #xpos_step
	scs
	adc #frame_width
	sta xpos_cnt
	.endp

	.proc frame_wrap_characters
	jsr scroll_characters

 	ldx char_index		
	inx
	cpx #char_n
	scc
	ldx #0
	stx char_index

 	.endp			;End of frame_wrap_characters
	.endp			;End of frame_wrap

	ldy #0
no_frame_wrap
	sty frame_cnt

	.endp			;End of scroll_players_left
	
	.proc fill_xpos_tab
	lda xpos_cnt
	ldx #$ff
	bne first
loop	sta xpos_tab,x
first	clc
	adc #xpos_step
	cmp #frame_width
	scc
	sbc #frame_width
	inx
	cpx #char_n
	bne loop
	.endp			;End of fill_xpos_tab

	.proc scroll_text_left

	ldx char_cnt
	inx
	cpx #char_width
	bne no_char_cnt_wrap

	ldx char_index
	inx
	cpx #char_n
	bne no_char_index_warp

	ldx #0
no_char_index_warp
	stx char_index

	mwa #bank1.effect.scroll.text.get_next_char animate_ptr
	jsr common.call_animate
	tay
	bne no_restart			;Zero means end of scroll phase
	
	lda stage_cnt			;Set stage end bit
	ora #$80
	sta stage_cnt
	rts

no_restart

	ldy #.len[data.charset]-1 	;Find character index in font
find_char_loop
	cmp data.charset,y
	beq char_found
	dey
	bpl find_char_loop
	bmi *				;Lock up at unknown character

char_found
	ldx char_index
	lda char_tab_offset,x		;Copy 7 lines
	tax
	.rept char_height
	lda data.font+#*data.font.line_size,y
	sta char_tab+#,x
	.endr

	ldx #0
no_char_cnt_wrap
 	stx char_cnt
   	.endp

	rts

;===============================================================

	.proc scroll_characters	;IN: char_index
	temp_char_index = volatile_zp
	temp_char_cnt = volatile_zp+1
	temp_line_cnt = volatile_zp+2

	mva char_index temp_char_index
	mva #char_n-2 temp_char_cnt
loop	ldx temp_char_index
	jsr copy_char
	ldx temp_char_index
	dex
	spl
	ldx #char_n-1
	stx temp_char_index

	dec temp_char_cnt
	bpl loop
	rts

	.proc copy_char		;IN: <X>=char_index
 	ldy char_tab_offset+1,x	;<Y> = target offset (higher address)
 	lda char_tab_offset,x	;<X> = source offset (lower address)
 	tax
 	mva #char_height-1 temp_line_cnt
loop	lda char_tab,x
	sta char_tab,y
	inx
	iny
	dec temp_line_cnt
	bpl loop
	rts
	.endp			;End of copy_char

	.endp			;End of scroll_characters

;===============================================================

	.local char_tab_offset
	.rept char_n
	.byte char_height*#
	.endr
	.rept 1			;Repeat for n+1 overflow
	.byte char_height*#
	.endr

	.endl

	.endp			;End of vblank

;===============================================================
	
	.proc playfield

	sta wsync
	ldx #1+4		;REF plus playerfield priority
	stx ctrlpf
	bit swchb
	smi
	ldx #0
	stx nusiz0		;Double width
	stx nusiz1
	lda #$f0
	sta pf0

	lda bank0.effect.sound.tt_cur_pat_index_c0
	asl
	asl
	asl
	asl
	sta animate_ptr
	lda #12
	ora animate_ptr
	sta colup0
	sta colup1

	ldx frame_cnt
	lda frame_addresses.lo,x
	sta animate_ptr
	lda frame_addresses.hi,x
	sta animate_ptr+1
	jmp common.call_animate		;Save 2 bytes of stack compared to JSR

	.local frame_addresses
	.local lo
	.use bank0
	.use bank2
	.use bank3

	.byte <effect.frames.frame_0,<effect.frames.frame_1,<effect.frames.frame_2,<effect.frames.frame_3,<effect.frames.frame_4,<effect.frames.frame_5
	.byte <effect.frames.frame_6,<effect.frames.frame_7,<effect.frames.frame_8,<effect.frames.frame_9,<effect.frames.frame_10,<effect.frames.frame_11
	.byte <effect.frames.frame_12,<effect.frames.frame_13

	.byte <effect.frames.frame_14, <effect.frames.frame_15,<effect.frames.frame_16,<effect.frames.frame_16,<effect.frames.frame_18,<effect.frames.frame_19
	.byte <effect.frames.frame_20,<effect.frames.frame_21,<effect.frames.frame_22,<effect.frames.frame_23,<effect.frames.frame_24,<effect.frames.frame_25
	.byte <effect.frames.frame_26, <effect.frames.frame_27

	.byte <effect.frames.frame_28,<effect.frames.frame_29
	.endl

	.local hi
	.byte >effect.frames.frame_0,>effect.frames.frame_1,>effect.frames.frame_2,>effect.frames.frame_3,>effect.frames.frame_4,>effect.frames.frame_5
	.byte >effect.frames.frame_6,>effect.frames.frame_7,>effect.frames.frame_8,>effect.frames.frame_9,>effect.frames.frame_10,>effect.frames.frame_11
	.byte >effect.frames.frame_12, >effect.frames.frame_13

	.byte >effect.frames.frame_14, >effect.frames.frame_15,>effect.frames.frame_16,>effect.frames.frame_16,>effect.frames.frame_18,>effect.frames.frame_19
	.byte >effect.frames.frame_20,>effect.frames.frame_21,>effect.frames.frame_22,>effect.frames.frame_23,>effect.frames.frame_24,>effect.frames.frame_25
	.byte >effect.frames.frame_26, >effect.frames.frame_27

	.byte >effect.frames.frame_28,>effect.frames.frame_29

	.endl
	
	m_info lo
	m_info hi
	.endl
	
	.endp			;End of playfield

;===============================================================

	.local data

	.local font
	line_size = 42
	ins "SV2K17-Scroll.chr"
	.endl

	.local charset
	.byte ' ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.!-''/'
	.endl
	
	.endl			;End of data

	.endp			;End of scroll

	m_info scroll
